@@ -5,5 +5,4 @@
     :copyright: © 2013 by Openlabs Technologies & Consulting (P) Limited
     :license: GPLv3, see LICENSE for more details.
 """
-from test_sugarcrm import suite
-
+from test_sugarcrm import suite     # noqa
